@@ -5,8 +5,11 @@ common parameter transformations, and pre-defined sets of parameter
 specifications.
 """
 
-from .sedmodel import ProspectorParams, SedModel, SpecModel, PolySpecModel, SplineSpecModel
+from .sedmodel import ProspectorParams, SedModel, SpecModel
+from .sedmodel import PolySpecModel, SplineSpecModel
+from .sedmodel import AGNSpecModel, LineSpecModel
 
-__all__ = ["ProspectorParams",
-           "SpecModel", "PolySpecModel", "SplineSpecModel",
+__all__ = ["ProspectorParams", "SpecModel",
+           "PolySpecModel", "SplineSpecModel",
+           "LineSpecModel", "AGNSpecModel",
            "SedModel"]
